/*    
 *    AddTagsToSam.cpp		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Kai Wang
 *
 *    Authors: Kai Wang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


//#define VS

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <map>
#include <stdlib.h>

#ifdef VS
#include <hash_map> //vc only
#include <hash_set> //vc only
#else
#include <ext/hash_map> //g++ only
#include <ext/hash_set> //g++ only
#endif

#ifdef VS
using namespace stdext;
#endif

#ifndef VS
using __gnu_cxx::hash;
using __gnu_cxx::hash_map;
using __gnu_cxx::hash_set;

namespace __gnu_cxx
{
	template<typename Traits, typename Allocator>
	struct hash<std::basic_string<char, Traits, Allocator> >
	{
		size_t operator()(const std::basic_string<char, Traits, Allocator>& __s) const
		{
			return __stl_hash_string(__s.c_str());
		}
	};
}

#endif

using namespace std;

size_t count_del = 0;

const size_t THIRTY_TWO = 32;
const size_t ALL_BITS_ON = static_cast<size_t>(-1);
const size_t LOWER_THIRTY_TWO_MASK = ALL_BITS_ON >> THIRTY_TWO;
const size_t UPPER_THIRTY_TWO_MASK = LOWER_THIRTY_TWO_MASK << THIRTY_TWO;

struct SamRec {

	string tag_name;

	size_t strand_t;

	string chrom_name;

	size_t start, end;

	unsigned short confid;

	string splice_way;

	string mapped_seq;

	unsigned short mis_match;

	vector<pair<size_t, int> > spliceway_vec;

	//vector<pair<size_t, int> > spliceway_vec;
	//bool wrong_format;

	size_t tagidx;

	//double filter_score;

	//size_t ave_intron_len;

	//double ave_junc_mis;

	//vector<string> junc_id;

	//int best;

	//string alters;

	string qual_str;

	//bool isunique, isexonic, isspliced, issmallins, issmalldel;
	
	//size_t canon_count, noncanon_count;

	//double canon_rate;

	int matched_id;

	char mate_match;

	size_t mate_offest;

	int mate_diff;

	string cur_line;

	size_t ih, hi;

	size_t mappedlen;

	char AS_strand; 

	//bool is_insert;

	//size_t mappedlen;

	//bool is_spliced;

	SamRec() {}

	SamRec(const string& tname, size_t strand, const string& cname, size_t st, unsigned short conf, const string& spliceway, const string& mapseq, unsigned short mismatch, size_t tidx, const string& qualstr, 
		char matematch, size_t mateoffest, int matediff, int min_del, const hash_map<string, hash_map<size_t, unsigned short> >& junc_hash, string line = "") : tag_name(tname), strand_t(strand), 
		chrom_name(cname), start(st), confid(conf), splice_way(spliceway), mapped_seq(mapseq), AS_strand(' '),
		mis_match(mismatch), matched_id(-1), mate_match(matematch), mate_offest(mateoffest), mate_diff(matediff), tagidx(tidx), qual_str(qualstr), cur_line(line), mappedlen(0)
	{
		size_t index = 0;

		string flag_str = " ";

		bool isdel = false;

		while (true)
		{
			if (index >= splice_way.length())
				break;

			int maplen;

			char flag;

			sscanf(splice_way.c_str() + index, "%d%c", &maplen, &flag);

			if (flag_str[0] == ' ')
			{
				if (flag == 'I')
					spliceway_vec.push_back(make_pair(start, -maplen));
				else if (flag == 'M')
				{
					spliceway_vec.push_back(make_pair(start, maplen));

					mappedlen += maplen;
				}
				else if (flag == 'N')
				{
					cout<<"start with N?"<<endl;
					spliceway_vec.push_back(make_pair(start + maplen, 0));
				}
			}
			else if (flag_str[0] == 'M')
			{
				if (flag == 'I')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second, -maplen));
				else if (flag == 'M')
				{
					cout << "continue Ms?"<<endl;
					spliceway_vec.back().second += maplen;

					mappedlen += maplen;
				}
				else if (flag == 'N')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second + maplen, 0));
			}
			else if (flag_str[0] == 'N')
			{
				if (flag == 'I')
					spliceway_vec.back().second = -maplen;
				else if (flag == 'M')
				{
					spliceway_vec.back().second = maplen;

					mappedlen += maplen;
				}
				else if (flag == 'N')
				{
					cout << "continue Ns?"<<endl;
					spliceway_vec.back().first += maplen;
				}
			}
			else if (flag_str[0] == 'I')
			{
				if (flag == 'I')
				{
					cout << "continue Is?"<<endl;
					spliceway_vec.back().second += -maplen;
				}
				else if (flag == 'M')
				{
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first, maplen));

					mappedlen += maplen;
				}
				else if (flag == 'N')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + maplen, 0));
			}

			flag_str[0] = flag;

			index = splice_way.find(flag_str, index) + 1;

			if (flag == 'N' && maplen <= min_del)
			{
				splice_way[index-1] = 'D';
				isdel = true;
			}
		}

		if (isdel)
			++count_del;

		if (spliceway_vec.size() > 1)
		{
			vector<pair<size_t, int> >::iterator vp_iter;

			size_t forward_count = 0, reverse_count = 0;

			for (vp_iter = spliceway_vec.begin(); vp_iter != spliceway_vec.end(); ++vp_iter)
			{
				size_t prefixst, prefixend, suffixst, prefixlen, suffixlen, combined_offset;

				//prefixst = spliceway_vec.begin()->first;

				if (vp_iter->second < 0)
				{
				}
				else if (vp_iter == spliceway_vec.end() - 1 || (vp_iter + 1)->second < 0)
				{
					//mappedlen += abs(vp_iter->second);
					//continue;
				}
				else
				{
					prefixst = vp_iter->first;

					prefixend = vp_iter->first + vp_iter->second - 1;

					suffixst = (vp_iter + 1)->first;// + spliceoutlen;

					prefixlen = vp_iter->second;

					suffixlen = (vp_iter + 1)->second;

					combined_offset = (prefixend << THIRTY_TWO) + suffixst;

					hash_map<string, hash_map<size_t, unsigned short> >::const_iterator hash_chrom_iter;

					hash_chrom_iter = junc_hash.find(chrom_name);

					if (hash_chrom_iter == junc_hash.end())
					{
						cout << "junction chromosome not find"<<endl;
						cout << this->tostring()<<endl;
						continue;
					}

					hash_map<size_t, unsigned short>::const_iterator hash_comb_iter;

					hash_comb_iter = hash_chrom_iter->second.find(combined_offset);

					if (hash_comb_iter == hash_chrom_iter->second.end())
					{
						cout << "junction not find"<<endl;
						cout << this->tostring()<<endl;
						continue;
					}

//ATAC 	1
//GTAT 	2
//CTGC 	3
//GCAG 	4
//GTAG 	5
//CTAC 	6
//others 0
					
					//cout << hash_comb_iter->second << endl;

					switch (hash_comb_iter->second) 
					{
					case 0:
						++forward_count;
						break;
					case 1:
						++forward_count;
						break;
					case 2:
						++reverse_count;
						break;
					case 3:
						++reverse_count;
						break;
					case 4:
						++forward_count;
						break;
					case 5:
						++forward_count;
						break;
					case 6:
						++reverse_count;
						break;
					default:
						cout << "none specified case: "<<hash_comb_iter->second << endl;
						break;
					}
				}
			}

			
			if (forward_count || reverse_count)
			{
				//cout << forward_count <<'\t' << reverse_count << endl;

				if (forward_count >= reverse_count && forward_count > 0)
					AS_strand = '+';
				else
					AS_strand = '-';
			}
		}
	}

	//SamRec(const string& tname, size_t strand, const string& cname, size_t st, size_t conf, const string& spliceway, const string& mapseq, unsigned short mismatch, size_t tidx, const string& alt, const string& qualstr, 
	//	char matematch, size_t mateoffest, int matediff, string line = "") : tag_name(tname), strand_t(strand), chrom_name(cname), start(st), confid(conf), splice_way(spliceway), mapped_seq(mapseq), 
	//	mis_match(mismatch), matched_id(-1), mate_match(matematch), mate_offest(mateoffest), mate_diff(matediff), wrong_format(false), tagidx(tidx), filter_score(0), best(0), alters(alt), qual_str(qualstr), 
	//	isunique(false), isexonic(false), isspliced(false), ave_intron_len(0), ave_junc_mis(-1), canon_count(0), noncanon_count(0), canon_rate(0), issmallins(false), issmalldel(false), cur_line(line),
	//	is_insert(false)/*, is_spliced(false)*/, mappedlen(0)
	//{
	//	//size_t index = 0;

	//	//if ((spliceway.find("I") != string::npos))
	//	//	issmallins = true;
	//	//else if ((spliceway.find("D") != string::npos))
	//	//	issmalldel = true;
	//	//else if (spliceway.find("N") != string::npos)
	//	//	isspliced = true;
	//	//else
	//	//	isexonic = true;		

	//	//string flag_str = " ";

	//	//isexonic = true;

	//	//isspliced = false;

	//	//while (true)
	//	//{
	//	//	if (index >= splice_way.length())
	//	//		break;

	//	//	int maplen;

	//	//	char flag;

	//	//	sscanf(splice_way.c_str() + index, "%d%c", &maplen, &flag);

	//	//	if (flag_str[0] == ' ')
	//	//	{
	//	//		if (flag == 'I')
	//	//			spliceway_vec.push_back(make_pair(start, -maplen));
	//	//		else if (flag == 'M')
	//	//		{
	//	//			spliceway_vec.push_back(make_pair(start, maplen));

	//	//			mappedlen += maplen;
	//	//		}
	//	//		else if (flag == 'N')
	//	//		{
	//	//			cout<<"start with N?"<<endl;
	//	//			spliceway_vec.push_back(make_pair(start + maplen, 0));
	//	//		}
	//	//	}
	//	//	else if (flag_str[0] == 'M')
	//	//	{
	//	//		if (flag == 'I')
	//	//			spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second, -maplen));
	//	//		else if (flag == 'M')
	//	//		{
	//	//			cout << "continue Ms?"<<endl;
	//	//			spliceway_vec.back().second += maplen;

	//	//			mappedlen += maplen;
	//	//		}
	//	//		else if (flag == 'N')
	//	//			spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second + maplen, 0));
	//	//	}
	//	//	else if (flag_str[0] == 'N')
	//	//	{
	//	//		if (flag == 'I')
	//	//			spliceway_vec.back().second = -maplen;
	//	//		else if (flag == 'M')
	//	//		{
	//	//			spliceway_vec.back().second = maplen;

	//	//			mappedlen += maplen;
	//	//		}
	//	//		else if (flag == 'N')
	//	//		{
	//	//			cout << "continue Ns?"<<endl;
	//	//			spliceway_vec.back().first += maplen;
	//	//		}
	//	//	}
	//	//	else if (flag_str[0] == 'I')
	//	//	{
	//	//		if (flag == 'I')
	//	//		{
	//	//			cout << "continue Is?"<<endl;
	//	//			spliceway_vec.back().second += -maplen;
	//	//		}
	//	//		else if (flag == 'M')
	//	//		{
	//	//			spliceway_vec.push_back(make_pair(spliceway_vec.back().first, maplen));

	//	//			mappedlen += maplen;
	//	//		}
	//	//		else if (flag == 'N')
	//	//			spliceway_vec.push_back(make_pair(spliceway_vec.back().first + maplen, 0));
	//	//	}

	//	//	flag_str[0] = flag;

	//	//	index = splice_way.find(flag_str, index) + 1;

	//	//}

	//	//end = spliceway_vec.back().first + spliceway_vec.back().second - 1;
	//}

	string tostring()
	{
		if (!cur_line.empty())
			return cur_line;

		char sam_rec_char[5000];

		if (AS_strand == ' ')
			sprintf(sam_rec_char, "%s\t%llu\t%s\t%llu\t%hu\t%s\t%c\t%llu\t%d\t%s\t%s\tNM:i:%hu\tIH:i:%llu\tHI:i:%llu", tag_name.c_str(), strand_t, 
			chrom_name.c_str(), start, confid, splice_way.c_str(), mate_match, mate_offest, mate_diff, mapped_seq.c_str(), qual_str.c_str(), mis_match, ih, hi);
		else
			sprintf(sam_rec_char, "%s\t%llu\t%s\t%llu\t%hu\t%s\t%c\t%llu\t%d\t%s\t%s\tNM:i:%hu\tIH:i:%llu\tHI:i:%llu\tXS:A:%c", tag_name.c_str(), strand_t, 
			chrom_name.c_str(), start, confid, splice_way.c_str(), mate_match, mate_offest, mate_diff, mapped_seq.c_str(), qual_str.c_str(), mis_match, ih, hi, AS_strand);

		return sam_rec_char;
	}
};

void read_junc(hash_map<string, hash_map<size_t, unsigned short> >& junc_hash, string junction_file)
{
	ifstream junc_ifs(junction_file.c_str());

	if (junc_ifs.is_open())
	{
		string skipline;
		getline(junc_ifs,skipline);

		while (!junc_ifs.eof() )
		{
			string line;
			getline(junc_ifs,line);
			if (line == "")
				continue;

			char chromname[100], juncname[100], strand[10], rgb[100], blocksoffset[1000];

			size_t juncst, juncend, prefixst, suffixend, kinds, hits, prefixlen, suffixlen;

			int flankcase;

			double entropy;

			//chr1    14757   15039   JUNC_2  521     +       14829   14970   255,0,0 2 72,70   0,212
			//chr1	3989698	3997737	JUNC_2	10	+	3989698	3997737	255,0,0	2	61,82,	0,8122,	1.22753	6	CTAC	0.999977	1	0	1	0.4
			//chr1	14829	14970	JUNC_1	3	-	14829	14970	255,0,0	2	92,57,	0,199,	1.09861	6	CTAC	1	1	0	2	1
			//chr1	14829	14970	JUNC_1	3	-	14829	14970	255,0,0	2	92,57,	0,199,	1.09861	6	CTAC	1	1	0	2	1
			// chromname,  &prefixst, &suffixend, juncname, &hits, strand, &juncst, &juncend, rgb, &kinds, &prefixlen, &suffixlen, blocksoffset, &entropy, &flankcase

			sscanf(line.c_str(), "%s\t%llu\t%llu\t%s\t%llu\t%s\t%llu\t%llu\t%s\t%llu\t%llu,%llu,\t%s\t%lf\t%d", chromname,  &prefixst, &suffixend, 
				juncname, &hits, strand, &juncst, &juncend, rgb, &kinds, &prefixlen, &suffixlen, blocksoffset, &entropy, &flankcase);

			string chromstr = chromname;

			size_t comb_offset = ((prefixst) << THIRTY_TWO) + (suffixend);

			//cout << blocksoffset << endl;
			//cout << entropy << endl;
			//cout << flankcase << endl;

			junc_hash[chromstr].insert(make_pair(comb_offset, flankcase));
		}
		junc_ifs.close();
	}
}

void read_chromo_size(string chrom_head, string chromosome_size_file)
{
	ifstream chromosome_size_file_ifs(chromosome_size_file.c_str());

	ofstream chrom_head_ofs(chrom_head.c_str());

	if (chromosome_size_file_ifs.is_open())
	{
		while (!chromosome_size_file_ifs.eof() )
		{
			string line;

			getline(chromosome_size_file_ifs, line);

			if (line == "")
				continue;

			char chromname[100];

			size_t chrom_size;

			sscanf(line.c_str(), "%s\t%llu", chromname,  &chrom_size);

			//@SQ     SN:chr16        LN:90354753

			chrom_head_ofs << "@SQ\tSN:"<<chromname<<"\tLN:"<<chrom_size <<endl;
		}

		chromosome_size_file_ifs.close();
	}
}


#define IS_PAIRED 0x0001
#define IS_PAIRED_MAPPED 0x0002
#define IS_UNMAPPED 0x0004
#define MATE_UNMAPPED 0x0008
#define IS_REVERSE 0x0010
#define IS_MATE_REVERSE 0x0020
#define IS_FIRST_END 0x040
#define IS_SECOND_END 0x0080
#define IS_PRIMARY 0x0100
#define IS_FAILED_QUAL_CHECK 0x0200
#define IS_PCR_DUP 0x0400

void
AddTagsToSam(const char* infile, const char* added_tags_file, const char* stat_file, size_t ispaired, hash_set<size_t>& reads_ids, int min_del, const hash_map<string, hash_map<size_t, unsigned short> >& junc_hash)
{
	ifstream infile_ifs(infile);

	string forward = added_tags_file; forward.append(".forward");

	ofstream forward_ofs(forward.c_str());

	string reverse = added_tags_file; reverse.append(".reverse");

	ofstream reverse_ofs(reverse.c_str());

	ofstream added_tags_file_ofs(added_tags_file);

	size_t prev_tag_idx = -1;

	//map<string, int> mapped_reads;

	vector<SamRec> mapped_reads;

	size_t count = 0, multiple_mapped = 0;

	if (infile_ifs.is_open())
	{
		string line;
		while (getline(infile_ifs,line))
		{
			if (line == "")
				continue;

			char tagname[1000], chrom[100], mapped[100], seq[1000], qual_str[1000];
			unsigned short strand, confd, mismatch;
			size_t offset;

			char mate_match;

			int mate_diff;

			size_t mate_offest, tagidx;

			//TRAN00000027662:59:252  16      chr14   56062712        255     8M157375N92M    *       0       0       TATTATTTTCCGCTTTCCCTGGGCTTACAGAGAATCCTTGCCCTTCTTGTACTGTGTCACTTTATGGGGTTGGTGCTTGCCACACTTCTTACAGAAAGTC 
			//#######$#%'*,*++*,/121222012122233455766655666555666677989:;<<=>>>>>==>??>>>>>>>>>>>>>>>>>>>=>=>>>>> NM:i:5  8:G>T,24:C>T,33:T>A,63:G>A,81:T>A

			int read_count = sscanf(line.c_str(), "%llu~%s\t%hu\t%s\t%llu\t%hu\t%s\t%c\t%llu\t%d\t%s\t%s\tNM:i:%hu", 
				&tagidx, tagname, &strand, chrom, &offset, &confd, mapped, &mate_match, &mate_offest, &mate_diff, seq, qual_str, &mismatch);

			if (string(tagname) == "UNC8-RDR3001640_17:1:61:16942:7395")
				cout <<"insert:"<< tagidx<<endl;
			//string tagnamestr = tagname;

			//size_t last_idx = tagnamestr.find_first_of("~");

			//size_t tagidx = -1;

			//if (last_idx != string::npos)
			//{
			//	string tagidxstr = tagnamestr.substr(0, last_idx);
			//	tagidx = atoi(tagidxstr.c_str()) - 1;

			//	reads_ids.insert(tagidx);
			//}
			//else
			//{
			//	cout << "read not indexed? "<<endl;
			//	continue;
			//}

			if (prev_tag_idx == -1 || prev_tag_idx == tagidx)
			{
				mapped_reads.push_back(SamRec(tagname, strand, chrom, offset, confd, mapped, seq, mismatch, tagidx, qual_str, mate_match, mate_offest, mate_diff, min_del, junc_hash));

				if (prev_tag_idx == -1)
					prev_tag_idx = tagidx;
			}
			else
			{
				vector<SamRec>::iterator msi_iter;

				++count;

				if (mapped_reads.size() > 1)
					++multiple_mapped;

				for (msi_iter = mapped_reads.begin(); msi_iter != mapped_reads.end(); ++msi_iter)
				{
					msi_iter->ih = mapped_reads.size();

					msi_iter->hi = msi_iter - mapped_reads.begin() + 1;

					msi_iter->strand_t |= ispaired;

					if (msi_iter->mate_match == '=')
						msi_iter->strand_t |= IS_PAIRED_MAPPED;

					char end_id = msi_iter->tag_name[msi_iter->tag_name.length() - 1];

					if (ispaired)
					{
						if (end_id == '1')
							msi_iter->strand_t |= IS_FIRST_END;
						else if (end_id == '2')
							msi_iter->strand_t |= IS_SECOND_END;
						else
						{
							cout <<"read has no end id ? "<<endl;
						}
					}

					if (msi_iter->strand_t & IS_REVERSE)
						reverse_ofs << msi_iter->tostring() << endl;
					else
						forward_ofs << msi_iter->tostring() << endl;

					//added_tags_file_ofs << msi_iter->tostring() << endl;


//#define IS_PAIRED 0x0001
//#define IS_PAIRED_MAPPED 0x0002
//#define IS_UNMAPPED 0x0004
//#define MATE_UNMAPPED 0x0008
//#define IS_REVERSE 0x0010
//#define IS_MATE_REVERSE 0x0020
//#define IS_FIRST_END 0x040
//#define IS_SECOND_END 0x0080
//#define IS_PRIMARY 0x0100
//#define IS_FAILED_QUAL_CHECK 0x0200
//#define IS_PCR_DUP 0x0400

				}

				reads_ids.insert(tagidx);

				mapped_reads.clear();

				mapped_reads.push_back(SamRec(tagname, strand, chrom, offset, confd, mapped, seq, mismatch, tagidx, qual_str, mate_match, mate_offest, mate_diff, min_del, junc_hash));

				prev_tag_idx = tagidx;
			}			
		}

		infile_ifs.close();
	}
	else
	{
		cout << "can't open file "<< infile<<endl; exit(1);
	}

	vector<SamRec>::iterator msi_iter;

	++count;

	if (mapped_reads.size() > 1)
		++multiple_mapped;

	if (prev_tag_idx != -1)
		reads_ids.insert(prev_tag_idx);

	for (msi_iter = mapped_reads.begin(); msi_iter != mapped_reads.end(); ++msi_iter)
	{
		msi_iter->ih = mapped_reads.size();

		msi_iter->hi = msi_iter - mapped_reads.begin() + 1;

		msi_iter->strand_t |= ispaired;

		if (msi_iter->mate_match == '=')
			msi_iter->strand_t |= IS_PAIRED_MAPPED;

		char end_id = msi_iter->tag_name[msi_iter->tag_name.length() - 1];

		if (ispaired)
		{
			if (end_id == '1')
				msi_iter->strand_t |= IS_FIRST_END;
			else if (end_id == '2')
				msi_iter->strand_t |= IS_SECOND_END;
			else
			{
				cout <<"read has no end id ? "<<endl;
			}
		}

		if (msi_iter->strand_t & IS_REVERSE)
			reverse_ofs << msi_iter->tostring() << endl;
		else
			forward_ofs << msi_iter->tostring() << endl;

		//added_tags_file_ofs << msi_iter->tostring() << endl;
	}

	ofstream stat_ofs(stat_file, ios_base::app);

	stat_ofs << "total mapped reads: " << count<<endl<<"multiple mapped: "<< multiple_mapped <<endl<< "deletion mapped: "<< count_del<<endl;;
}

size_t AddUnmappedReadsFQ(const hash_set<size_t>& reads_ids, string reads_file, string unmapped_reads_sam)
{
	ifstream ifs(reads_file.c_str());

	ofstream ofs(unmapped_reads_sam.c_str());

	string unmapped_reads_sam_index = unmapped_reads_sam;

	unmapped_reads_sam_index.append(".indexed");

	ofstream ofs_indexed(unmapped_reads_sam_index.c_str());

	size_t read_count = 0, count = 0;

	if (ifs.is_open())
	{
		while (!ifs.eof() )
		{
			string reads_name, reads_seq, reads_qual, strand;

			getline(ifs, reads_name);

			if (reads_name.empty())
				continue;

			if (reads_name[reads_name.length() - 1] == '\r')
				reads_name = reads_name.substr(0, reads_name.length() - 1);

			getline(ifs, reads_seq);

			if (reads_seq.empty())
				continue;

			if (reads_seq[reads_seq.length() - 1] == '\r')
				reads_seq = reads_seq.substr(0, reads_seq.length() - 1);

			getline(ifs, strand);

			if (strand.empty())
				continue;

			if (strand[strand.length() - 1] == '\r')
				strand = strand.substr(0, strand.length() - 1);

			getline(ifs, reads_qual);

			if (reads_qual.empty())
				continue;

			if (reads_qual[reads_qual.length() - 1] == '\r')
				reads_qual = reads_qual.substr(0, reads_qual.length() - 1);

			++read_count;

			reads_name = reads_name.substr(1);

			size_t real_index;

			sscanf(reads_name.c_str(), "%llu", &real_index);

			//if (real_index != read_count)
			//{
			//	cout << "index not match"<<endl<<reads_name << "\t"<<read_count << endl;
			//	getchar();
			//}

			if (reads_ids.find(real_index) == reads_ids.end()) // unmapped
			{
				size_t last_idx = reads_name.find_first_of("~");

				if (last_idx != string::npos)
				{
					reads_name = reads_name.substr(last_idx + 1);
				}

				//if (reads_name == "UNC8-RDR3001640_17:1:61:16942:7395")
				//	cout <<"read_count:"<< read_count << endl;

				ofs << reads_name <<"\t4\t*\t0\t0\t*\t*\t0\t0\t"<<reads_seq <<"\t"<<reads_qual <<"\tIH:i:0\tHI:i:0" <<endl;

				ofs_indexed <<real_index<<'~' <<reads_name <<"\t4\t*\t0\t0\t*\t*\t0\t0\t"<<reads_seq <<"\t"<<reads_qual <<"\tIH:i:0\tHI:i:0" <<endl;

				++count;
			}
		}
	}

	return count;
}

size_t AddUnmappedReadsFA(const hash_set<size_t>& reads_ids, string reads_file, string unmapped_reads_sam)
{
	ifstream ifs(reads_file.c_str());

	ofstream ofs(unmapped_reads_sam.c_str());

	string unmapped_reads_sam_index = unmapped_reads_sam;

	unmapped_reads_sam_index.append(".indexed");

	ofstream ofs_indexed(unmapped_reads_sam_index.c_str());

	size_t read_count = 0, count = 0;

	if (ifs.is_open())
	{
		while (!ifs.eof() )
		{
			string reads_name, reads_seq, reads_qual, strand;

			getline(ifs, reads_name);

			if (reads_name.empty())
				continue;

			if (reads_name[reads_name.length() - 1] == '\r')
				reads_name = reads_name.substr(0, reads_name.length() - 1);

			getline(ifs, reads_seq);

			if (reads_seq.empty())
				continue;

			if (reads_seq[reads_seq.length() - 1] == '\r')
				reads_seq = reads_seq.substr(0, reads_seq.length() - 1);

			++read_count;

			reads_name = reads_name.substr(1);

			size_t real_index;

			sscanf(reads_name.c_str(), "%llu", &real_index);

			if (reads_ids.find(real_index) == reads_ids.end()) // unmapped
			{
				size_t last_idx = reads_name.find_first_of("~");

				if (last_idx != string::npos)
				{
					reads_name = reads_name.substr(last_idx + 1);
				}

				ofs << reads_name <<"\t4\t*\t0\t0\t*\t*\t0\t0\t"<<reads_seq <<"\t"<<string(reads_seq.length(), 'I') <<"\tIH:i:0\tHI:i:0" <<endl;

				ofs_indexed <<real_index<<'~' <<reads_name <<"\t4\t*\t0\t0\t*\t*\t0\t0\t"<<reads_seq <<"\t"<<string(reads_seq.length(), 'I') <<"\tIH:i:0\tHI:i:0" <<endl;

				++count;
			}
		}
	}

	return count;
}

int main(int argc, char** argv)
{
	if (argc < 5)
	{
		cout << "infile added_tags_file stat_file 0|1(is_paired) reads_file unmapped_samfile fq|fa" <<endl;
		exit(0);
	}
	const char* infile = argv[1];

	const char* added_tags_file = argv[2];

	const char* stat_file = argv[3];

	size_t ispaired = atoi(argv[4]);

	string reads_file = argv[5];

	string unmapped_sam_file = argv[6];

	string reads_format = argv[7];

	int min_del = atoi(argv[8]);

	char* junction_file = argv[9];

	char* chromo_size_file = argv[10];

	string chrom_head = added_tags_file; chrom_head.append(".head");

	read_chromo_size(chrom_head, chromo_size_file);

	bool isfq = (reads_format == "fq" ? true : false);

	hash_set<size_t> reads_ids;

	AddUnmappedReadsFQ(reads_ids, reads_file, unmapped_sam_file);

	hash_map<string, hash_map<size_t, unsigned short> > junc_hash;

	cout << "read junctions"<< endl;

	read_junc(junc_hash, junction_file);

	cout << "add tag to sam "<<endl;

	AddTagsToSam(infile, added_tags_file, stat_file, ispaired, reads_ids, min_del, junc_hash);

	size_t unmapped_reads;

	cout << "output unmapped sam "<<endl;

	if (isfq)
		unmapped_reads = AddUnmappedReadsFQ(reads_ids, reads_file, unmapped_sam_file);
	else
		unmapped_reads = AddUnmappedReadsFA(reads_ids, reads_file, unmapped_sam_file);

	cout << count_del <<" deletions alignments"<<endl;

}
