/*    
 *    check_reads_format.cpp		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Kai Wang
 *
 *    Authors: Kai Wang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include <stdio.h>
#include <iostream>
#include <string>
#include <fstream>
#include <stdlib.h>
#include <errno.h>

using namespace std;

size_t minlen = 10000, maxlen = 0;

size_t readlen = 0;

int countall=0;

void convert_ex_paired_sep(char* extendFile, char* extendFile2, int fasta)
{
	size_t read_id = 0;

	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}

	ifstream input_fs2(extendFile2);
	if( !input_fs2 ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}   

	string readidline;
	string line, line2, line3, line4;
	string line5, line6, line7, line8;
	
	while(!input_fs.eof() && !input_fs2.eof())
	{
		++countall;

		getline(input_fs, line);
		getline(input_fs, line2);

		if (line.empty() || line2.empty())
			continue;

		//output_fs <<line[0] <<countall<<'~'<<  line.c_str() + 1 << endl << line2 << endl;

		if (!fasta)
		{
			getline(input_fs, line3);	
			getline(input_fs, line4);

			if (line3.empty() || line4.empty())
				continue;

			//output_fs << line3 << endl << line4 << endl;
		}

		++countall;

		getline(input_fs2, line5);	
		getline(input_fs2, line6);

		if (line5.empty() || line6.empty())
			continue;

		//output_fs <<line[0]  <<countall<<'~'<< line5.c_str() + 1 << endl << line6 << endl;

		if (!fasta)
		{
			getline(input_fs2, line7);	
			getline(input_fs2, line8);

			if (line7.empty() || line8.empty())
				continue;

			//output_fs << line7 << endl << line8 << endl;
		}

		++read_id;

		if (line.substr(line.length() - 2, 2) != "/1" && line.substr(line.length() - 2, 2) != "/2")
		{
			cerr << "pairend read name not end with /1 or /2"<<endl;
			cerr << "the " << read_id << "th read in "<< extendFile<< endl;
			cerr << line<<endl;

			exit(1);
		}

		if (line5.substr(line5.length() - 2, 2) != "/1" && line5.substr(line5.length() - 2, 2) != "/2")
		{
			cerr << "pairend read name not end with /1 or /2"<<endl;
			cerr << "the " << read_id << "th read in "<< extendFile2 << endl;
			cerr << line5<<endl;

			exit(1);
		}		

		if (line.substr(0, line.length() - 1) != line5.substr(0, line5.length() - 1))
		{
			cerr << "base name of two ends not consistent"<<endl;
			cerr << "the " << read_id << "th read in "<< extendFile<< " and " << extendFile2 << endl;
			cerr << line << endl << line5<<endl;

			exit(1);
		}

		if (line2.length() < readlen)
		{
			cerr << "read length < specified read length"<<endl;
			cerr << "the " << read_id << "th read in " << extendFile << endl;
			cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

			exit(2);
		}

		if (line6.length() < readlen)
		{
			cerr << "read length < specified read length"<<endl;
			cerr << "the " << read_id << "th read in " << extendFile2 << endl;
			cerr << line5 << endl << line6 << endl << line7 << endl << line8<<endl;
			exit(2);
		}

		if (!fasta)
		{
			if (line2.length() != line4.length())
			{
				cerr << "read length and quality string length not consistent"<<endl;
				cerr << "the " << read_id << "th read in " << extendFile << endl;
				cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

				exit(3);
			}

			if (line6.length() != line8.length())
			{
				cerr << "read length and quality string length not consistent"<<endl;
				cerr << "the " << read_id << "th read in " << extendFile2 << endl;
				cerr << line5 << endl << line6 << endl << line7 << endl << line8<<endl;

				exit(3);
			}
		}

		if (line2.length() > maxlen)
			maxlen = line2.length();

		if (line2.length() < minlen)
			minlen = line2.length();

		if (line6.length() > maxlen)
			maxlen = line6.length();

		if (line6.length() < minlen)
			minlen = line6.length();
	}

	input_fs.close();
	
	cout<<countall<<" reads converted for "<<extendFile<<endl; 
}

void convert_ex_paired_comb(char* extendFile, int fasta)
{
	size_t read_id = 0;

	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}

	string readidline;
	string line, line2, line3, line4;
	string line5, line6, line7, line8;
	
	while(!input_fs.eof())
	{
		++countall;

		getline(input_fs, line);
		getline(input_fs, line2);

		if (line.empty() || line2.empty())
			continue;

		//output_fs <<line[0] <<countall<<'~'<<  line.c_str() + 1 << endl << line2 << endl;

		if (!fasta)
		{
			getline(input_fs, line3);	
			getline(input_fs, line4);

			if (line3.empty() || line4.empty())
				continue;

			//output_fs << line3 << endl << line4 << endl;
		}

		++countall;

		getline(input_fs, line5);	
		getline(input_fs, line6);

		if (line5.empty() || line6.empty())
			continue;

		//output_fs <<line[0]  <<countall<<'~'<< line5.c_str() + 1 << endl << line6 << endl;

		if (!fasta)
		{
			getline(input_fs, line7);	
			getline(input_fs, line8);

			if (line7.empty() || line8.empty())
				continue;

			//output_fs << line7 << endl << line8 << endl;
		}

		++read_id;

		if (line.substr(line.length() - 2, 2) != "/1" && line.substr(line.length() - 2, 2) != "/2")
		{
			cerr << "pairend read name not end with /1 or /2"<<endl;
			cerr << "the " << read_id * 2 - 1 << "th read in "<< extendFile<< endl;
			cerr << line<<endl;

			exit(1);
		}

		if (line5.substr(line5.length() - 2, 2) != "/1" && line5.substr(line5.length() - 2, 2) != "/2")
		{
			cerr << "pairend read name not end with /1 or /2"<<endl;
			cerr << "the " <<  read_id * 2 << "th read in "<< extendFile << endl;
			cerr << line5<<endl;

			exit(1);
		}

		if (line.substr(0, line.length() - 1) != line5.substr(0, line5.length() - 1))
		{
			cerr << "base name of two ends not consistent"<<endl;
			cerr << "the " << read_id * 2 - 1 << "th read in "<< extendFile<< " and " << read_id * 2 << "th read in" << extendFile << endl;
			cerr << line << endl << line5<<endl;

			exit(1);
		}

		//cout << line2.length() << ':' << readlen << endl;

		if (line2.length() < readlen)
		{
			cerr << "read length < specified read length"<<endl;
			cerr << "the " << read_id * 2 - 1 << "th read in " << extendFile << endl;
			cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

			exit(2);
		}

		if (line6.length() < readlen)
		{
			cerr << "read length < specified read length"<<endl;
			cerr << "the " << read_id * 2 << "th read in " << extendFile << endl;
			cerr << line5 << endl << line6 << endl << line7 << endl << line8<<endl;

			exit(2);
		}

		if (!fasta)
		{
			if (line2.length() != line4.length())
			{
				cerr << "read length and quality string length not consistent"<<endl;
				cerr << "the " << read_id * 2 - 1 << "th read in " << extendFile << endl;
				cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

				exit(3);
			}

			if (line6.length() != line8.length())
			{
				cerr << "read length and quality string length not consistent"<<endl;
				cerr << "the " << read_id * 2 << "th read in " << extendFile << endl;
				cerr << line5 << endl << line6 << endl << line7 << endl << line8<<endl;

				exit(3);
			}
		}

		if (line2.length() > maxlen)
			maxlen = line2.length();

		if (line2.length() < minlen)
			minlen = line2.length();

		if (line6.length() > maxlen)
			maxlen = line6.length();

		if (line6.length() < minlen)
			minlen = line6.length();
	}

	input_fs.close();
	
	cout<<countall<<" reads converted for "<<extendFile<<endl; 
}

void convert_ex_single(char* extendFile, int fasta)
{
	size_t read_id = 0;

	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}

	string readidline;
	string line, line2, line3, line4;
	
	while(!input_fs.eof())
	{
		++countall;

		getline(input_fs, line);
		getline(input_fs, line2);

		if (line.empty() || line2.empty())
			continue;

		//output_fs <<line[0] <<countall<<'~'<<  line.c_str() + 1 << endl << line2 << endl;

		if (!fasta)
		{
			getline(input_fs, line3);	
			getline(input_fs, line4);

			if (line3.empty() || line4.empty())
				continue;

			//output_fs << line3 << endl << line4 << endl;
		}

		++read_id;

		if (line2.length() < readlen)
		{
			cerr << "read length < specified read length"<<endl;
			cerr << "the " << read_id << "th read in " << extendFile << endl;
			cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

			exit(2);
		}

		if (!fasta)
		{
			if (line2.length() != line4.length())
			{
				cerr << "read length and quality string length not consistent"<<endl;
				cerr << "the " << read_id * 2 - 1 << "th read in " << extendFile << endl;
				cerr << line << endl << line2 << endl << line3 << endl << line4 <<endl;

				exit(3);
			}
		}

		if (line2.length() > maxlen)
			maxlen = line2.length();

		if (line2.length() < minlen)
			minlen = line2.length();
	}

	input_fs.close();
	
	cout<<countall<<" reads converted for "<<extendFile<<endl; 
}

int main(int argc, char* argv[])
{
	if (argc < 4)
	{
		fprintf(stderr,"error: too few arguments\n");
		fprintf(stderr,"reads_1 reads_2 ... read_len ispaired(1|0) isfasta\n");
		exit(1);
	}

	int fasta = atoi(argv[argc - 1]);

	int pairend = atoi(argv[argc - 2]);

	readlen = atoi(argv[argc - 3]);

	cout << "readlen:" << readlen << endl;

	if (pairend && argc % 2 == 0)
	{
		for (size_t i = 1; i < argc - 3; i += 2)
		{
			convert_ex_paired_sep(argv[i], argv[i + 1], fasta);
		}
	}
	else if (pairend && argc % 2 != 0)
	{
		for (size_t i = 1; i < argc - 3; i += 1)
		{
			convert_ex_paired_comb(argv[i], fasta);
		}
	}
	else
	{
		for (size_t i = 1; i < argc - 3; ++i)
			convert_ex_single(argv[i], fasta);
	}

	cout <<"maxlen "<<maxlen<<" minlen "<<minlen<<endl;

	return 0;
}
