/*    
 *    bwtmap_file.h		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Zeng Zheng
 *
 *    Authors: Zeng Zheng
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef BWTMAP_FILE_H
#define BWTMAP_FILE_H

#include <string.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <vector>
#include <map>

using namespace std;

class Bwtmap_File
{
public:
	Bwtmap_File(const char* filename)
	{
		bwt_fs.open(filename);
		if( !bwt_fs ) 
		{
			fprintf(stderr,"error: open bwtmap file %s error\n",filename);
			exit(1);
		}  
		endoffile=false;
		pairend=false;
	}

	Bwtmap_File(const char* filename, bool pair_end)
	{
		bwt_fs.open(filename);
		if( !bwt_fs ) 
		{
			fprintf(stderr,"error: open bwtmap file %s error\n",filename);
			exit(1);
		}  
		endoffile=false;
		pairend=pair_end;
	}

	~Bwtmap_File()
	{

	}

	bool load_next_line()
	{
		if(line.empty())
		{
			if( getline(bwt_fs, line))
				return true;
			else
			{
				endoffile=true;			
				return false;
			}
		}
		else
			return true;
	}

	int load_next_block(vector<Bwtmap_Info>& read_mapping_set) // load a block of junctions
	{
		read_mapping_set.clear();
		if(!load_next_line())
		{	
			return -1;
		}
		Bwtmap_Info first_bwt_info(line, pairend);
		read_mapping_set.push_back(first_bwt_info);
		line.clear();
		while(load_next_line())
		{
			Bwtmap_Info new_bwt_info(line, pairend);
			if(new_bwt_info.read_id==first_bwt_info.read_id)
			{
				read_mapping_set.push_back(new_bwt_info);
				line.clear();
			}
			else
			{
				break;
			}
		}
		return (int)read_mapping_set.size();
	}

	void close()
	{
		bwt_fs.close();
	}

	ifstream bwt_fs;
	string line;
	bool endoffile;
	bool pairend;
};

#endif

